/*****************************************************************************
 *
 * Copyright (c) 2020, Marcel Schoppmeier
 * Westfälische Wilhelms-Universität Münster, AG Embedded Systems 
 * All rights reserved.
 *
 * This file is part of SysCSecT
 *
 * SysCSecT is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * SysCSecT is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SysCSecT.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Please report any problems or bugs to: sysc.sec@uni-muenster.de
 *
 ****************************************************************************/

#ifndef SUBJECT
#define SUBJECT

#include "constants.h"
#include "Utils.cpp"
#include "RSA.cpp"

/*
 * The subject gets messages and perform the square-and-multiply algorithm
 * 
 * @param name name of the suject
 */

SC_MODULE(Subject)   
{
    //ports for sc_fifo channel
    sc_fifo_in<bigint> in;
    sc_fifo_out<bigint> out;
    
    sc_out<bool> cipherProceeded_out;//port for sc_signal channel
    
    bigint y = 0; //message for the timeanalyzer
    

    /* 
     * It starts the program, receives the data and calculates the squaringAlgorithm
     */
    inline void subject_main(){
        out->write(y);//Instruction for the timeanalyzer to start sending messages
        
        while(true){
            wait(in->data_written_event());
            bigint msg = in->read();
           //Debug: cout << "received:" << msg <<endl;
            bigint result = squaringAlgorithm(msg);
           //Debug: cout << "calculated:" << result <<endl;
            cipherProceeded_out.write(!cipherProceeded_out->read());//Booleanswap. Sends notification when message has been processed for the sensitive list of the timeanalyzer.
        }
        
    }
    
    SC_HAS_PROCESS(Subject);//associate process with the module class
    //constructor
	Subject(sc_module_name name): sc_module(name), cipherProceeded_out("cPi"), in("in"), out("out"){ 
        SC_THREAD(subject_main);
    }
};

#endif
