/*****************************************************************************
 *
 * Copyright (c) 2020, Marcel Schoppmeier
 * Westfälische Wilhelms-Universität Münster, AG Embedded Systems 
 * All rights reserved.
 *
 * This file is part of SysCSecT
 *
 * SysCSecT is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * SysCSecT is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SysCSecT.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Please report any problems or bugs to: sysc.sec@uni-muenster.de
 *
 ****************************************************************************/


#ifndef PARAMETERS_H
#define PARAMETERS_H

#include <systemc.h>
#include <cmath> //for the pow function

/*
 * This file is used to set global parameters for the execution.
*/

//Path to the folder in which the results are saved
#define RESULTPATH std::string("/home/herber/git/es-code/TimingAttack/results")

//parameters:
#define MAXBITS             32                      //Maximum number of bits a random number may consist of. Values greater than 32 must not be set due to the data type int.
#define MAXBITINT           pow(2,MAXBITS/2)        //Highest number that the random number generator may generate. Number was deliberately chosen smaller to speed up the execution. Maximum number does not lead to significantly different results (pow(2,MAXBITS-1) - 1 = 2147483647).
#define VALUE_N             12345678910111213       //Modulo for the RSA procedure. A high modulo leads to better deviations of the timing models.
#define NUMBER_OF_MESSAGES  500                 //Number of messages sent to the target.

//The interval of private keys to be tested:
#define FROM_NUMBER 0
#define TO_NUMBER 256 //the result chart is designed for values up to 260, but can be extended in the tex file




#define GUESSBITS 2//Number of bits to be guessed. 1 or 2 are valid entries.

/*Variants:
 * 1 - mulitplication time depends only on the highest significant bit of x and it depends not on the intermediate values s (only for 1 bit)
 * 2 - mulitplication time depends on the highest significant bit also for the intermediate values s
 * 3 - mulitplication time depends on number of bits during calculation
 * 4 - mulitplication time depends on the calculation effort for the Booth algorithm (only for 2 bits)
 */
#define VARIANT 3
    
//constant time operations
    #define SHIFT   6
    #define MOD     8
    #define ASSIGN  9
    
/*
 * number of times the timing attack procedure should be carried out for a single key in order to analyse whether there are different results for several attempts.
 * To print the results for the texfile NUMBER_OF_EXECTUTIONS must be 1.
 */
#define NUMBER_OF_EXECUTIONS 1
        
typedef sc_bigint<MAXBITS>  bigint;
    
#endif
