/*****************************************************************************
 *
 * Copyright (c) 2020, Marcel Schoppmeier
 * Westfälische Wilhelms-Universität Münster, AG Embedded Systems 
 * All rights reserved.
 *
 * This file is part of SysCSecT
 *
 * SysCSecT is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 *
 * SysCSecT is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SysCSecT.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 *  Please report any problems or bugs to: sysc.sec@uni-muenster.de
 *
 ****************************************************************************/


#include "constants.h"
#include "Timeanalyser.cpp"
#include "Subject.cpp"
#include "Utils.cpp"

/*
 * main method
*/

int sc_main(int argc, char* argv[])
{
    //check for invalid parameters in constant.h
    if(Utils::invalidParameters())
        return EXIT_FAILURE;
    
    //primitive FIFO channels
    sc_fifo<bigint> subjectFifo;
    sc_fifo<bigint> analyserFifo;

    //primitive boolean channel
    sc_signal<bool> cipherProceededSig;
    
    //initialization
    Timeanalyser t("Timeanalyser", NUMBER_OF_MESSAGES);
    Subject subject("Subject");
    
    //interconnections
    subject.out(analyserFifo);
    subject.in(subjectFifo);
    subject.cipherProceeded_out(cipherProceededSig);
    t.out(subjectFifo);
    t.in(analyserFifo);
    t.cipherProceeded_in(cipherProceededSig);

    //start program
    sc_start();
    
  return EXIT_SUCCESS;
}
